#!/usr/bin/env python
import urllib.error
import urllib.parse
import urllib.request
from itertools import count, groupby
from os import path
import tempfile

from lxml import etree


def convertToRanges(L):
    G = (list(x) for _, x in groupby(L, lambda x, c=count(): next(c) - x))
    return ["-".join(map(str, (g[0], g[-1])[:len(g)])) for g in G]


class XmlPdbParser:
    def __init__(self, pdbcode, chain, work_dir=tempfile.gettempdir(), localfile=False, atom="CA"):
        self.pdb = pdbcode
        self.chain = chain
        self.atom = atom
        self.work_dir = work_dir
        self.localfile = localfile
        self.xmlgzfile = self.download_if_not_exist()
        self.root = etree.parse(self.xmlgzfile, etree.XMLParser(ns_clean=True)).getroot()
        self.NS = self.root.nsmap

    def download_if_not_exist(self):
        pdb_xml_file = path.join(self.work_dir, self.pdb + '.xml.gz')
        if not path.isfile(pdb_xml_file) or path.getsize(pdb_xml_file) == 0:
            f = urllib.request.urlopen('https://www.rcsb.org/pdb/files/' + self.pdb + '.xml.gz')
            fw = open(pdb_xml_file, "wb")
            fw.write(f.read())
            fw.close()
            f.close()
        return pdb_xml_file


class getCoordinates(XmlPdbParser):
    def __init__(self, pdbcode, work_dir=tempfile.gettempdir(), localfile=False, atom="CA"):
        super().__init__(pdbcode, None, work_dir, localfile, atom=atom)
        l = self.root.xpath(
            "//PDBx:atom_siteCategory/PDBx:atom_site[PDBx:auth_atom_id=\"" + atom + "\"][PDBx:pdbx_PDB_model_num='1'][PDBx:group_PDB='ATOM']/PDBx:auth_asym_id",
            namespaces=self.NS)
        self.chains = set()
        self.chains.update(e.text for e in l)

    def getChainIndexes(self):
        return self.chains

    def getCalfa(self, chain, output='', preserve_seqid=False):
        l = self.root.xpath(
            "//PDBx:atom_siteCategory/PDBx:atom_site[not(PDBx:label_alt_id/text()) or PDBx:label_alt_id='A'][PDBx:auth_atom_id=\"" + self.atom + "\"][PDBx:pdbx_PDB_model_num='1'][PDBx:group_PDB='ATOM' or (PDBx:group_PDB='HETATM' and PDBx:label_comp_id='MSE') or (PDBx:group_PDB='HETATM' and PDBx:label_comp_id='ORN') or (PDBx:group_PDB='HETATM' and PDBx:label_comp_id='PCA') or (PDBx:group_PDB='HETATM' and PDBx:label_comp_id='DGL')][PDBx:auth_asym_id='" + chain + "']",
            namespaces=self.NS)
        # mozna by pewnie zrobic inaczej or hetatom and (xxx or xxx or xxx ....)
        o = ''
        # r = []
        prev_seqid = 999999999999999
        first_resid = -1
        for e in l:
            d = e.find("PDBx:label_seq_id", namespaces=self.NS)
            if d.text:
                first_resid = int(d.text)
                break
        prev_seqid = first_resid - 9999999999999
        for e in l:
            x = e.find("PDBx:Cartn_x", namespaces=self.NS)
            y = e.find("PDBx:Cartn_y", namespaces=self.NS)
            z = e.find("PDBx:Cartn_z", namespaces=self.NS)
            seqid = e.find("PDBx:label_seq_id", namespaces=self.NS)
            if not seqid.text:
                continue
            if int(seqid.text) == prev_seqid:
                prev_seqid = int(seqid.text)
                print(prev_seqid)
                continue
            prev_seqid = int(seqid.text)
            l = [seqid, x, y, z]
            if None not in l:
                if preserve_seqid:
                    new_seqid = int(l[0].text)
                else:
                    new_seqid = int(l[0].text) - first_resid + 1
                o += "%4d %8.3f %8.3f %8.3f\n" % (new_seqid, float(l[1].text), float(l[2].text), float(l[3].text))
        if output != '':
            f = open(output, "w")
            f.write(o)
            f.close()
        else:
            return o

    def getCalfaPdbFormat(self, chain, output='', preserve_seqid=False):
        l = self.root.xpath(
            "//PDBx:atom_siteCategory/PDBx:atom_site[not(PDBx:label_alt_id/text()) or PDBx:label_alt_id='A'][PDBx:auth_atom_id=\"" + self.atom + "\"][PDBx:pdbx_PDB_model_num='1'][PDBx:group_PDB='ATOM' or (PDBx:group_PDB='HETATM' and PDBx:label_comp_id='MSE') or (PDBx:group_PDB='HETATM' and PDBx:label_comp_id='ORN') or (PDBx:group_PDB='HETATM' and PDBx:label_comp_id='PCA') or (PDBx:group_PDB='HETATM' and PDBx:label_comp_id='DGL')][PDBx:auth_asym_id='" + chain + "']",
            namespaces=self.NS)
        o = ''
        r = []
        prev_seqid = 999999999999999
        first_resid = -1
        for e in l:
            d = e.find("PDBx:label_seq_id", namespaces=self.NS)
            if d.text:
                first_resid = int(d.text)
                break
        prev_seqid = first_resid - 9999999999999

        for e in l:
            x = e.find("PDBx:Cartn_x", namespaces=self.NS)
            y = e.find("PDBx:Cartn_y", namespaces=self.NS)
            z = e.find("PDBx:Cartn_z", namespaces=self.NS)
            seqname = e.find("PDBx:label_comp_id", namespaces=self.NS)
            seqid = e.find("PDBx:label_seq_id", namespaces=self.NS)
            true_seqid = e.find("PDBx:auth_seq_id", namespaces=self.NS)
            if not seqid.text:
                continue
            if preserve_seqid and not true_seqid.text:
                continue
            if int(seqid.text) == prev_seqid:
                prev_seqid = int(seqid.text)
                continue
            prev_seqid = int(seqid.text)
            bfactor = e.find("PDBx:B_iso_or_equiv", namespaces=self.NS)
            l = (seqid, seqname, chain, seqid, x, y, z, bfactor)
            if None not in l:
                if preserve_seqid:
                    new_seqid = int(true_seqid.text)
                else:
                    new_seqid = int(seqid.text) - first_resid + 1
                l = (new_seqid, str(seqname.text), str(chain), new_seqid, float(x.text), float(y.text), float(z.text),
                     float(bfactor.text))
                if self.atom == 'CA':
                    o += "ATOM%7d  CA%5s%2s%4d%12.3f%8.3f%8.3f  1.00%6.2f           C\n" % l
                else:
                    o += "ATOM%7d  C3'%4s%2s%4d%12.3f%8.3f%8.3f  1.00%6.2f           C\n" % l
        if output != '':
            f = open(output, "w")
            f.write(o)
            f.close()
        else:
            return o


class fetchPDBinfo(XmlPdbParser):
    def __init__(self, pdbcode, chain='A', work_dir=tempfile.gettempdir(), localfile=False, atom="CA"):
        super().__init__(pdbcode, chain, work_dir, localfile, atom=atom)
        l = self.root.xpath(
            "//PDBx:atom_siteCategory/PDBx:atom_site[PDBx:auth_atom_id=\"" + atom + "\"][PDBx:pdbx_PDB_model_num='1'][PDBx:group_PDB='ATOM']/PDBx:auth_asym_id",
            namespaces=self.NS)
        self.chains = set()
        self.ordered_chains = []
        for e in l:
            if e is not None and e.text not in self.ordered_chains:
                self.chains.update(e.text)
                self.ordered_chains.append(e.text)
        self.codification = {"ALA": 'A',
                             "CYS": 'C',
                             "ASP": 'D',
                             "GLU": 'E',
                             "PHE": 'F',
                             "GLY": 'G',
                             "HIS": 'H',
                             "ILE": 'I',
                             "LYS": 'K',
                             "LEU": 'L',
                             "MET": 'M',
                             "MSE": 'M',
                             "ASN": 'N',
                             "PYL": 'O',
                             "PRO": 'P',
                             "GLN": 'Q',
                             "ARG": 'R',
                             "SER": 'S',
                             "THR": 'T',
                             "SEC": 'U',
                             "VAL": 'V',
                             "TRP": 'W',
                             "5HP": 'E',
                             "ABA": 'A',
                             "AIB": 'A',
                             "BMT": 'T',
                             "CEA": 'C',
                             "CGU": 'E',
                             "CME": 'C',
                             "CRO": 'X',
                             "CSD": 'C',
                             "CSO": 'C',
                             "CSS": 'C',
                             "CSW": 'C',
                             "CSX": 'C',
                             "CXM": 'M',
                             "DAL": 'A',
                             "DAR": 'R',
                             "DCY": 'C',
                             "DGL": 'E',
                             "DGN": 'Q',
                             "DHI": 'H',
                             "DIL": 'I',
                             "DIV": 'V',
                             "DLE": 'L',
                             "DLY": 'K',
                             "DPN": 'F',
                             "DPR": 'P',
                             "DSG": 'N',
                             "DSN": 'S',
                             "DSP": 'D',
                             "DTH": 'T',
                             "DTR": 'X',
                             "DTY": 'Y',
                             "DVA": 'V',
                             "FME": 'M',
                             "HYP": 'P',
                             "KCX": 'K',
                             "LLP": 'K',
                             "MLE": 'L',
                             "MVA": 'V',
                             "NLE": 'L',
                             "OCS": 'C',
                             "ORN": 'A',
                             "PCA": 'E',
                             "PTR": 'Y',
                             "SAR": 'G',
                             "SEP": 'S',
                             "STY": 'Y',
                             "TPO": 'T',
                             "TPQ": 'F',
                             "TYS": 'Y',
                             "TYR": 'Y'}
        self._getCAmissing()

    def getCalfaBreaks(self, preserve_seqid=False):
        l = self.root.xpath(
            "//PDBx:atom_siteCategory/PDBx:atom_site[not(PDBx:label_alt_id/text()) or PDBx:label_alt_id='A'][PDBx:auth_atom_id=\"" + self.atom + "\"][PDBx:pdbx_PDB_model_num='1'][PDBx:group_PDB='ATOM' or (PDBx:group_PDB='HETATM' and PDBx:label_comp_id='MSE') or (PDBx:group_PDB='HETATM' and PDBx:label_comp_id='ORN') or (PDBx:group_PDB='HETATM' and PDBx:label_comp_id='PCA') or (PDBx:group_PDB='HETATM' and PDBx:label_comp_id='DGL')][PDBx:auth_asym_id='" + self.chain + "']",
            namespaces=self.NS)
        # mozna by pewnie zrobic inaczej or hetatom and (xxx or xxx or xxx ....)
        o = []
        # r = []
        prev_seqid = 999999999999999
        first_resid = -1
        for e in l:
            d = e.find("PDBx:label_seq_id", namespaces=self.NS)
            if d is not None and d.text:
                first_resid = int(d.text)
                break
        prev_seqid = first_resid - 9999999999999
        for e in l:
            x = e.find("PDBx:Cartn_x", namespaces=self.NS)
            y = e.find("PDBx:Cartn_y", namespaces=self.NS)
            z = e.find("PDBx:Cartn_z", namespaces=self.NS)
            seqid = e.find("PDBx:label_seq_id", namespaces=self.NS)
            if not seqid.text:
                continue
            if int(seqid.text) == prev_seqid:
                prev_seqid = int(seqid.text)
                print(prev_seqid)
                continue
            prev_seqid = int(seqid.text)
            l = [seqid, x, y, z]
            if None not in l:
                if preserve_seqid:
                    new_seqid = int(l[0].text)
                else:
                    new_seqid = int(l[0].text) - first_resid + 1
                o.append((new_seqid, float(l[1].text), float(l[2].text), float(l[3].text)))
        # check chain breaks
        eps = 4.2 * 4.2
        brk = []
        for i in range(1, len(o)):
            i1 = o[i - 1][0]
            i2 = o[i][0]
            x1 = o[i - 1][1]
            y1 = o[i - 1][2]
            z1 = o[i - 1][3]

            x2 = o[i][1]
            y2 = o[i][2]
            z2 = o[i][3]

            d = (x1 - x2) ** 2 + (y2 - y1) ** 2 + (z1 - z2) ** 2
            if d > eps:
                brk.append(i1)
                brk.append(i2)
        return brk

    def getFirstResidueIndex(self):
        return 1

    def getTrueFirstResidueIndex(self):
        l = self.root.xpath(
            "//PDBx:atom_siteCategory/PDBx:atom_site[not(PDBx:label_alt_id/text()) or PDBx:label_alt_id='A'][PDBx:auth_atom_id=\"" + self.atom + "\"][PDBx:pdbx_PDB_model_num='1'][PDBx:group_PDB='ATOM' or (PDBx:group_PDB='HETATM' and PDBx:label_comp_id='MSE') or (PDBx:group_PDB='HETATM' and PDBx:label_comp_id='ORN') or (PDBx:group_PDB='HETATM' and PDBx:label_comp_id='PCA') or (PDBx:group_PDB='HETATM' and PDBx:label_comp_id='DGL')][PDBx:auth_asym_id='" + self.chain + "']/PDBx:auth_seq_id",
            namespaces=self.NS)
        for e in l:
            if e.text:
                # print e.text
                return int(e.text)
        return 1  # could be not true anymore

    def getSeqOneLetterCode(self):
        l = self.root.xpath(
            "//PDBx:atom_siteCategory/PDBx:atom_site[PDBx:auth_atom_id=\"" + self.atom + "\"][PDBx:auth_asym_id='" + self.chain + "'][PDBx:pdbx_PDB_model_num='1'][PDBx:group_PDB='ATOM' or (PDBx:group_PDB='HETATM' and PDBx:label_comp_id='MSE') or (PDBx:group_PDB='HETATM' and PDBx:label_comp_id='ORN') or (PDBx:group_PDB='HETATM' and PDBx:label_comp_id='PCA') or (PDBx:group_PDB='HETATM' and PDBx:label_comp_id='DGL')][not(PDBx:label_alt_id/text()) or PDBx:label_alt_id='A']/PDBx:label_comp_id",
            namespaces=self.NS)
        k = list(self.codification.keys())
        seq = ""
        for e in l:
            s = e.text
            if self.atom == 'CA':
                if s in k:
                    seq += self.codification[s]
                else:
                    seq += "X"
            else:
                seq += s
        return seq

    def getMissingArray(self):
        return self.missing_array

    def _getCAmissing(self):
        seq = []
        l = self.root.xpath(
            "//PDBx:atom_siteCategory/PDBx:atom_site[PDBx:auth_atom_id=\"" + self.atom + "\"][PDBx:auth_asym_id='" + self.chain + "'][PDBx:pdbx_PDB_model_num='1'][PDBx:group_PDB='ATOM' or (PDBx:group_PDB='HETATM' and PDBx:label_comp_id='MSE') or (PDBx:group_PDB='HETATM' and PDBx:label_comp_id='ORN') or (PDBx:group_PDB='HETATM' and PDBx:label_comp_id='PCA') or (PDBx:group_PDB='HETATM' and PDBx:label_comp_id='DGL')][not(PDBx:label_alt_id/text()) or PDBx:label_alt_id='A']/PDBx:label_seq_id",
            namespaces=self.NS)
        for e in l:
            if e.text: seq.append(int(e.text))
        if len(seq) > 0:  # TODO make better function for getting missing - it fails e.g. in case of 3bpd_C
            first = seq[0]
        for e in range(len(seq)):
            seq[e] = seq[e] - first + 1  # przenumerowanie od 1 z zachowaniem przerw

        self.crystlen = len(seq)
        if len(seq) > 0:
            self.missing = [x for x in range(seq[0], seq[-1] + 1) if x not in seq]
            self.missing_array = self.missing
            self.missing = convertToRanges(self.missing)
            self.seq_idx = seq
            self.seq_len = seq[-1]
        else:
            self.missing = []
            self.missing_array = self.missing
            self.seq_len = 0

    def getSeqLength(self):
        return self.seq_len

    def getPdbCreationDate(self):
        self.date = self.root.xpath(
            "//PDBx:pdbx_database_statusCategory/PDBx:pdbx_database_status/PDBx:recvd_initial_deposition_date",
            namespaces=self.NS)[0].text
        #        self.date = self.root.xpath("//PDBx:database_PDB_revCategory/PDBx:database_PDB_rev/PDBx:date_original",namespaces=self.NS)[0].text
        try:
            return self.date
        except:
            return None

    def getCAlen(self):
        return self.crystlen

    def getMissing(self):
        return self.missing

    def setChain(self, chain):
        self.chain = chain
        self._getCAmissing()

    def getChains(self):
        return self.chains

    def getOrderedChains(self):
        return self.ordered_chains

    def getPubtitlePubmed(self):
        pubmed = self.root.xpath("//PDBx:citationCategory/PDBx:citation[1]/PDBx:pdbx_database_id_PubMed",
                                 namespaces=self.NS)
        doi = self.root.xpath("//PDBx:citationCategory/PDBx:citation[1]/PDBx:pdbx_database_id_DOI", namespaces=self.NS)
        title = self.root.xpath("//PDBx:citationCategory/PDBx:citation[1]/PDBx:title", namespaces=self.NS)
        desc = self.root.xpath("//PDBx:structCategory/PDBx:struct/PDBx:title", namespaces=self.NS)
        src = self.root.xpath("//PDBx:entity_src_genCategory/PDBx:entity_src_gen/PDBx:pdbx_gene_src_scientific_name",
                              namespaces=self.NS)
        key = self.root.xpath("//PDBx:struct_keywordsCategory/PDBx:struct_keywords/PDBx:pdbx_keywords",
                              namespaces=self.NS)
        molecutag = self.root.xpath("//PDBx:entityCategory/PDBx:entity/PDBx:pdbx_description", namespaces=self.NS)

        doi = doi[0].text if doi else None
        molecutag = molecutag[0].text if molecutag else None
        key = key[0].text.capitalize() if key else None
        src = src[0].text if src else None
        pubmed = pubmed[0].text if pubmed else None
        title = title[0].text if title else None
        desc = desc[0].text.capitalize() if desc else None

        return (doi, pubmed, desc, title, src, key, molecutag)


if __name__ == "__main__":
    #    pdb=argv[1].lower()
    r = fetchPDBinfo('6CZR', '1a', '/tmp', localfile=False, atom="C3'")
    # r = fetchPDBinfo('/tmp','1j85', 'A', localfile=False, atom="CA")
    print(r.getSeqLength())
    print(r.getCAlen())
    # b = getCoordinates("/tmp", "1uak")
    # b.getCalfaPdbFormat("A", output=path.join("/tmp", "t.pdb"))
    # a = fetchPDBinfo("/tmp", "1uak","A")
    # print(a.getCalfaBreaks())
    # sequence = a.getSeqOneLetterCode()
    # print("seq",sequence)
    # str_len = int(a.getCAlen())
    # print("str_len",str_len)
    # seq_len = a.getSeqLength()
    # print("seq_len",seq_len)
    # missing = ", ".join(a.getMissing())
    # print("miss",missing)
    # date = a.getPdbCreationDate()
    # print("date",date)
    # structure_start = a.getFirstResidueIndex()
    # print("start",structure_start)
    # doi, pubmed, pubtitle, prottitle, source, keyword, molname =  a.getPubtitlePubmed()
    # print("rest","\n".join([doi, pubmed, pubtitle, prottitle, source, keyword, molname]))



import warnings

from spyprot import getCath
from spyprot.fetchPDBinfo import fetchPDBinfo
from spyprot.fetchAnnotations import PfamAnnotation, ECAnnotation


# def test_get_annotations():
#     a = getAnnotations("1j85", "A")
#     assert str(a.get()[0]) =="['1j85', 'A', '2', '142', 'PF00588.18', 'SpoU_methylase', 'SpoU rRNA Methylase family', '4.3E-32']"


def test_get_ec_annotations():
    a = ECAnnotation("1cak", "A")
    assert str(a.get()) == "[('4.2.1.1', 'Carbonic anhydrase.')]"


def test_get_pfam_annotations():
    a = PfamAnnotation("1j85", "A")
    assert str(a.get()) == "[('1j85', 'A', 'SpoU_methylase', 'SpoU rRNA Methylase family', 'PF00588', 'K31', 100)]"
    a = PfamAnnotation("3bdg", "B")
    assert str(a.get()) == "[('3bdg', 'B', 'Alk_phosphatase', 'Alkaline phosphatase', 'PF00245', 'S31', 3.5)]"


def test_fetch_pdb_info():
    from Bio import BiopythonExperimentalWarning
    import tempfile
    with warnings.catch_warnings():
        warnings.simplefilter('ignore', BiopythonExperimentalWarning)
        a = fetchPDBinfo("1uak", "A")
        assert str(a.getCalfaBreaks()) == '[162, 172]'


def test_cddf_parser():
    d = getCath("1j85", "A")
    assert d[0]['DOMAIN'] == '1j85A00' and d[0]['CATHCODE'] == '3.40.1280.10' and d[0]['CLASS'] == 'Alpha Beta'



import os
import tempfile

from spyprot.ilbsm_database_downloader import ILBSMDatabaseDownloader


def test_knotprot_download():
    SEARCH_STRING = 'https://knotprot.cent.uw.edu.pl/browse/?set=True&bridgeType=probab&knotTypes=-31&array=0&raw=1'
    URL_KNOT_MATRIX = 'https://knotprot.cent.uw.edu.pl/static/knot_data/{0}/{1}/{0}_{1}.png'
    URL_KNOTOID_SUB_RAW = 'https://knotprot.cent.uw.edu.pl/static/knot_data//{0}/{1}/{0}_{1}_data.txt'
    URL_CHAIN_XYZ = 'https://knotprot.cent.uw.edu.pl/chains/{0}/{1}/chain.xyz.txt'
    with tempfile.TemporaryDirectory() as out_dir:
        dd = ILBSMDatabaseDownloader(SEARCH_STRING, [URL_KNOT_MATRIX,
                                                     URL_KNOTOID_SUB_RAW,
                                                     URL_CHAIN_XYZ], out_dir, create_separate_dirs=False)
        dd.get_all()
        assert len(os.listdir(out_dir)) >= 96


def test_lassoprot_download():
    SEARCH_STRING = 'https://lassoprot.cent.uw.edu.pl/browse/?lassoType=L3&set=0&bridgeType=ssbridge%2Camide%2Cester%2Cthioester%2Cothers&array=0&raw=1'
    URL_ALL_FILES = 'https://lassoprot.cent.uw.edu.pl/files/{0}_{1}_all.tar.bz2'
    with tempfile.TemporaryDirectory() as out_dir:
        dd = ILBSMDatabaseDownloader(SEARCH_STRING, [URL_ALL_FILES], out_dir, create_separate_dirs=False)
        dd.get_all()
        assert len(os.listdir(out_dir)) >= 53


def test_genus_download():
    SEARCH_STRING = 'https://genus.fuw.edu.pl/browse/?moltag=hydrolase%2Fpeptide&set=True&is_rna=&raw=1'
    URL_BONDS_PROTEIN = 'https://genus.fuw.edu.pl/file/{0}/{1}/{0}_{1}.chimera'
    with tempfile.TemporaryDirectory() as out_dir:
        dd = ILBSMDatabaseDownloader(SEARCH_STRING, [URL_BONDS_PROTEIN], out_dir, create_separate_dirs=False)
        dd.get_all()
        assert len(os.listdir(out_dir)) >= 95




import pytest

from spyprot.fetchChainInfo import SimilarChains, UniqueChains, IdenticalChains, ReleasedPDBs, SearchException
from datetime import datetime


def test_fetchChainInfo_IdenticalChains():
    f = IdenticalChains('6wm4', 'Y')
    assert str(f.get()) == "['X', 'Y', 'Z']"


def test_fetchChainInfo_UniqueChains():
    f = UniqueChains('6wm4')
    assert str(sorted(f.get())) == str(
        sorted(['R', 'O', 'H', 'K', 'S', 'T', 'A', 'D', 'X', 'G', 'N', 'U', 'V', '0', '1', 'Q', 'P']))


def test_fetchChainInfo_UniqueChains_2():
    f = UniqueChains('2jlo')
    assert f.get() == []


def test_fetchChainInfo_UniqueChains_3():
    f = UniqueChains('6zj3', only_rna=True)
    assert str(sorted(f.get())) == str(sorted(
        ['S1', 'S2', 'S3', 'S4', 'S5', 'LA', 'LB', 'LC', 'LD', 'LE', 'LF', 'LG', 'LH', 'LI', 'LJ', 'LK', 'LL', 'LM',
         'LN', 'LO']))
    f = UniqueChains('6zj3', only_prot=True)
    assert len(f.get()) == 78
    f = UniqueChains('6zj3', only_prot=False, only_rna=False)
    assert len(f.get()) == 98


def test_fetchChainInfo_SimilarChains():
    sim = SimilarChains(pdb='1j85', chain='A')
    assert str(
        sim.get()) == "[('1J85', 'A'), ('3E5Y', 'A'), ('3N4J', 'A'), ('3N4K', 'A'), ('4JAK', 'A'), ('4JAL', 'A'), ('4KDZ', 'A'), ('4KGN', 'A'), ('4PZK', 'A'), ('5CO4', 'A'), ('6AHW', 'A'), ('6QH8', 'A'), ('6QKV', 'A'), ('7D51', 'A')]"
    sim = SimilarChains('6lt7', 'A', identity=30)
    assert len(sim.get()) >= 3
    sim = SimilarChains(pdb='7css', chain='A')
    assert len(sim.get()) == 0


def test_fetchChainInfo_SimilarChains_fail():
    with pytest.raises(SearchException):
        sim = SimilarChains(seq='AFHAGAGOANBAG')
        sim.get()
    with pytest.raises(SearchException):
        sim = SimilarChains(pdb='1j85aa', chain='A')
        sim.get()
    with pytest.raises(SearchException):
        sim = SimilarChains(pdb='7djq', chain='C')
        sim.get()


def test_fetchChainInfo_ReleasedProteins():
    from_date = datetime(2020, 11, 10).date()
    to_date = datetime(2020, 11, 18).date()
    res = ReleasedPDBs(from_date, to_date).get()
    assert str(res).startswith(
        "[('5rw2', 'A'), ('5rw3', 'A'), ('5rw4', 'A'), ('5rw5', 'A'), ('5rw6', 'A'), ('5rw7', 'A'), ('5rw8', 'A'), ('5rw9', 'A'), ('5rwa', 'A'),")


def test_fetchChainInfo_ReleasedProteins_2():
    from_date = "2020-11-18"
    res = ReleasedPDBs(from_date).get()
    assert len(res) == 489
    assert str(res).startswith(
        "[('6hpj', 'B'), ('6kml', 'A'), ('6kml', 'B'), ('6kmq', 'A'), ('6kmq', 'B'), ('6l9k', 'A'), ('6l9k', 'Q'), ('6l9l', 'A'), ('6l9l', 'B'), ('6l9l', 'C'),")
    res = ReleasedPDBs(from_date, only_rna=True).get()
    assert str(res) == "[('6hpj', 'A'), ('6vem', 'A'), ('6wbr', 'B'), ('6wc0', 'B'), ('6wvj', 'R')]"
    res = ReleasedPDBs(from_date, only_rna=False, only_prot=False).get()
    assert len(res) == 494



# source   jcollado http://stackoverflow.com/questions/1191374/subprocess-with-timeout?rq=1
from os import SEEK_END
import io
import subprocess
from subprocess import TimeoutExpired
import threading
# source https://stackoverflow.com/questions/4789837/how-to-terminate-a-python-subprocess-launched-with-shell-true
import psutil


def kill(proc_pid):
    process = psutil.Process(proc_pid)
    for proc in process.children(recursive=True):
        proc.kill()
    process.kill()


class Command(object):
    '''Run Command in the Operating System

       cmd: string - command to run
       '''

    def __init__(self, cmd):
        self.cmd = cmd
        self.process = None
        self.o = None
        self.e = None
        self.rcode = -100
        self.logfile = None
        self.logerrfile = None

    def run(self, timeout, logfile=None):
        '''Run the command provided in constructor

           Parameters
           ==========
           timeout: int - (seconds) - how long to wait before killing the command
           logfile: string - path to file to which to write the std out from running command

           '''
        self.logfile = logfile

        def target():
            try:
                if logfile:
                    logfile_handle = open(logfile, 'w+')
                    self.logerrfile = logfile + ".err"
                    logerrfile_handle = open(self.logerrfile, 'w+')
                    self.process = subprocess.Popen(self.cmd, shell=True, stdout=logfile_handle,
                                                    stderr=logerrfile_handle)
                else:
                    self.process = subprocess.Popen(self.cmd, shell=True, stdout=subprocess.PIPE,
                                                    stderr=subprocess.PIPE)
                self.o, self.e = self.process.communicate(timeout=timeout)
                if logfile:
                    logfile_handle.flush()
                    logfile_handle.close()
                    logerrfile_handle.flush()
                    logerrfile_handle.close()
            except TimeoutExpired:
                print("timeout occured for process pid: " + str(self.process.pid))
                self.process.terminate()
                self.rcode = -400
            except OSError as e:
                print("OS Error: " + str(e))
                self.rcode = -100
                return

        thread = threading.Thread(
            target=target)  # thread, not process - to easily comuniacate with PIPE and return rcode.
        thread.start()

        thread.join(timeout)
        if thread.is_alive():
            # self.process.terminate() # is stoping main process not those run from it - then thread.join() is waiting for shell (and other children) to end!
            # os.killpg(os.getpgid(self.process.pid), signal.SIGKILL) # Send the signal to all the process groups - including this one which is wating for next thread.join()...
            kill(
                self.process.pid)  # using psutil, but could be written without it - searching in processes for children.
            thread.join()
            raise Exception('Processing binary file has been terminated by timeout. Error? Loop?')
        if self.process:
            self.rcode = self.process.returncode

    def getReturnCode(self):
        return self.rcode

    def getOut(self):
        if self.o:
            return self.o.decode('utf8')
        elif self.logfile:
            with io.open(self.logfile, 'r', encoding="utf8", errors='ignore') as f:
                return "\n".join(line.strip() for line in f.read().splitlines())
        else:
            return None

    def getError(self):
        if self.e:
            return self.e.decode('utf8')
        elif self.logerrfile:
            with io.open(self.logerrfile, 'r', encoding="utf8", errors='ignore') as f:
                print("\n".join(line.strip() for line in f.read().splitlines()))
        else:
            return None

    def getTrimmedOut(self):
        if self.logfile:
            with io.open(self.logfile, 'r', encoding="utf8", errors='ignore') as f:
                try:
                    f.seek(-2000, SEEK_END)
                except:
                    pass
                return "..." + "\n".join(line.strip() for line in f.read().splitlines())
        if self.o is None:
            return None
        out = self.getOut()
        if out and len(out) > 500:
            return "..." + out[-500:]
        else:
            return out

    def getCmd(self):
        return self.cmd


if __name__ == "__main__":
    command = Command("sleep 2; echo 'Process finished'; sleep  2; echo 'gsgsg'; echo 'sdgsg'; echo 'sagwrgsg'")
    command.run(timeout=15, logfile="/tmp/log")
    print("Got out: " + command.getTrimmedOut())
    command.run(timeout=3, logfile="/tmp/log.2")
    print(command.getOut())
    print('BYE')



from skbuild import setup
from setuptools import find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="spyprot",
    version='0.6.4',
    author="INTERDISCIPLINARY LABORATORY of BIOLOGICAL SYSTEMS MODELLING, University of Warsaw, Warsaw, Poland",
    author_email="bmjastrzebski@gmail.com, p.rubach@cent.uw.edu.pl",
    description="This package provides a set of tools for accessing protein databases and manipulating PDB/CIF files.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/ilbsm/spyProt",
    packages=find_packages(),
    install_requires=[
        'tqdm>=4.31.1', 'lxml>=4.5.0', 'requests>=2.0.0', 'biopython>1.60', 'psutil>5.6.0', 'subprocess32>=3.5.0', 'wget>=3.0', 'mysolr>=0.8.3'
    ],
    python_requires='>=3.6.0',
    classifiers=[
        "Programming Language :: Python :: 3",
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent"
    ],
)
